package cn.com.qmth.examcloud.config.center.core;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.PropertiesUtil;
import cn.com.qmth.examcloud.web.bootstrap.BootstrapSecurityUtil;
import com.google.common.collect.Sets;
import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.SpringApplication;
import org.springframework.context.ConfigurableApplicationContext;

import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;

/**
 * 配置中心启动器
 *
 * @author WANGWEI
 * @date 2019年3月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class ConfigCenterBootstrap {

    private static ExamCloudLog log = ExamCloudLogFactory.getLog(ConfigCenterBootstrap.class);

    /**
     * 启动配置中心
     */
    public static ConfigurableApplicationContext run(Class<?> primarySource, String... args) {

        Properties props = new Properties();
        PropertiesUtil.loadFromResource("application.properties", props);

        String active = null;
        String secretKey = null;
        if (null != args) {
            for (String s : args) {
                s = s.trim();
                if (s.startsWith("--examcloud.startup.secretKey=")) {
                    secretKey = s.substring(s.indexOf("=") + 1);
                }
                if (s.startsWith("--spring.profiles.active=")) {
                    active = s.substring(s.indexOf("=") + 1);
                }
            }
        }

        if (null == active) {
            String value = props.getProperty("spring.profiles.active");
            if (StringUtils.isNotBlank(value)) {
                active = value.trim();
            }
        }
        log.info("active=" + active);

        if (StringUtils.isBlank(active)) {
            log.error("[spring.profiles.active] value must not empty!");
            System.exit(-1);
        }

        if (StringUtils.isBlank(secretKey)) {
            throw new RuntimeException("Secret key must not empty!");
        }

        PropertiesUtil.loadFromResource("application-" + active + ".properties", props);
        props.setProperty("spring.profiles.active", active);

        String test = props.getProperty("$$.examcloud.test");
        if (StringUtils.isBlank(test)) {
            log.error("[$$.examcloud.test] value must not empty!");
            System.exit(-1);
        }

        try {
            BootstrapSecurityUtil.decrypt(test.trim(), secretKey);
        } catch (Exception e) {
            throw new RuntimeException("Secret key is wrong!");
        }

        try {
            BootstrapSecurityUtil.decrypt(props, secretKey);
        } catch (Exception e) {
            log.error("Secret key decrypt fail!");
            System.exit(-1);
        }

        Set<String> argSet = Sets.newLinkedHashSet();
        for (Entry<Object, Object> p : props.entrySet()) {
            String arg = "--" + p.getKey() + "=" + p.getValue();
            argSet.add(arg);
        }

        String[] newArgs = argSet.toArray(new String[argSet.size()]);

        BootstrapSecurityManager.getInstance().setSecretKey(secretKey);
        BootstrapSecurityManager.getInstance().setActive(active);

        ConfigurableApplicationContext context = null;
        try {
            context = SpringApplication.run(primarySource, newArgs);
        } catch (Exception e) {
            log.error("Project startup fail!", e);
            System.exit(-1);
        }

        return context;
    }

}
